#include <admodel.h>

#include <df1b2fun.h>

#include <adrndeff.h>

  extern "C"  {
    void ad_boundf(int i);
  }
#include <t.htp>

  df1b2_parameters * df1b2_parameters::df1b2_parameters_ptr=0;
  model_parameters * model_parameters::model_parameters_ptr=0;
model_data::model_data(int argc,char * argv[]) : ad_comm(argc,argv)
{
  nsteps.allocate("nsteps");
  k.allocate("k");
  a.allocate(1,k+1,"a");
  freq.allocate(1,k,"freq");
 sum_freq=sum(freq);
}

model_parameters::model_parameters(int sz,int argc,char * argv[]) : 
 model_data(argc,argv) , function_minimizer(sz)
{
  model_parameters_ptr=this;
  initializationfunction();
  log_tau.allocate(-14,15,2,"log_tau");
  log_nu.allocate(-15,4,"log_nu");
  beta.allocate(.1,1.0,-1,"beta");
  log_sigma.allocate(-5,3,"log_sigma");
  tau.allocate("tau");
  nu.allocate("nu");
  sigma.allocate("sigma");
  S.allocate(1,k+1,"S");
  #ifndef NO_AD_INITIALIZE
    S.initialize();
  #endif
  f.allocate("f");  /* ADOBJECTIVEFUNCTION */
  u.allocate(1,k+1,"u");
}

void model_parameters::initializationfunction(void)
{
  log_tau.set_initial_value(0);
  beta.set_initial_value(0.6666667);
  log_nu.set_initial_value(0);
  log_sigma.set_initial_value(-2);
}
void model_parameters::userfunction(void)
{
  tau=exp(log_tau);
  nu=exp(log_nu);
  sigma=exp(log_sigma);
   dvariable Integral;
   int i;
   for (i=1;i<=k+1;i++)
   {
     a_index=i;
     Integral=adromb(-3.0,3.0,nsteps);
     S(i)=Integral;
     //S(i)=h(0);
   }
   f=0.0;
   for (i=1;i<=k;i++)
   {
     dvariable ff=0.0;
     dvariable diff=posfun((S(i)-S(i+1))/S(i),.000001,ff);
     f-=freq(i)*log(1.e-50+S(i)*diff);
     f+=ff;
   }
   f+=sum_freq*log(1.e-50+S(1));
   f+=0.5*norm2(u);
}

dvariable model_parameters::adromb(double a,double b,int ns)
{
  const double base = 4;
  const int JMAX=50;
  dvariable ss=0.0;
  int MAXN = min(JMAX, ns);
  dvar_vector s(1,MAXN+1);
  for(int j=1; j<=MAXN+1; j++)
  {
    s[j] = trapzd(a,b,j,ss);
  }
  for(int iter=1; iter<=MAXN+1; iter++)
  {
    for(int j=1; j<=MAXN+1-iter; j++)
    {
      s[j] = (pow(base,iter)*s[j+1]-s[j])/(pow(base,iter)-1);
    }
  }
  return s[1];
}

dvariable model_parameters::trapzd(double a,double b,int n,const dvariable& s)
{
  double x,num_interval,hn;
  dvariable sum;
 // static dvariable s;
  static int interval;
  int j;
  model_parameters * ptr= (model_parameters *) mycast();
  if (n == 1) {
    interval=1;
    return (s=0.5*(b-a)*(h(a)+h(b)));
  } else {
    num_interval=interval;
    hn=(b-a)/num_interval;
    x=a+0.5*hn;
    for (sum=0.0,j=1;j<=interval;j++,x+=hn) sum += h(x);
    interval *= 2;
    s=0.5*(s+(b-a)*sum/num_interval);
    return s;
  }
}

dvariable model_parameters::h(const double z)
{
  dvariable tmp;
  tmp=exp(-.5*z*z + u(a_index)+ tau*(-1.+exp(-nu*pow(a(a_index),beta)*exp(sigma*z))) );  
  return tmp;
}

void model_parameters::report()
{
 adstring ad_tmp=initial_params::get_reportfile_name();
  ofstream report((char*)(adprogram_name + ad_tmp));
  if (!report)
  {
    cerr << "error trying to open report file"  << adprogram_name << ".rep";
    return;
  }
  report << "nsteps = " << std::setprecision(10) <<  nsteps << endl;
  report << "f = " << std::setprecision(10) <<  f << endl;
  report << "a" << endl << a << endl;
  report << "freq" << endl << freq << endl;
  report << "S" << endl << S << endl;
  report << "S/S(1)" << endl << std::ios::fixed << std::setprecision(6) << S/S(1) << endl;
  report << "tau "  << tau << endl; 
  report << "nu "  << nu << endl; 
  report << "beta "  << beta << endl; 
}
  long int arrmblsize=0;

int main(int argc,char * argv[])
{
  ad_set_new_handler();
  ad_exit=&ad_boundf;
    gradient_structure::set_NO_DERIVATIVES();
    gradient_structure::set_YES_SAVE_VARIABLES_VALUES();
  #if defined(__GNUDOS__) || defined(DOS386) || defined(__DPMI32__)  || \
     defined(__MSVC32__)
      if (!arrmblsize) arrmblsize=150000;
  #else
      if (!arrmblsize) arrmblsize=25000;
  #endif
    df1b2variable::noallocate=1;
    df1b2_parameters mp(arrmblsize,argc,argv);
    mp.iprint=10;

    function_minimizer::random_effects_flag=1;
    df1b2variable::noallocate=0;
    mp.preliminary_calculations();
    mp.computations(argc,argv);
    return 0;
}

extern "C"  {
  void ad_boundf(int i)
  {
    /* so we can stop here */
    exit(i);
  }
}

void model_parameters::preliminary_calculations(void){
  admaster_slave_variable_interface(*this);
  }

model_data::~model_data()
{}

model_parameters::~model_parameters()
{}

void model_parameters::final_calcs(void){}

void model_parameters::set_runtime(void){}

#ifdef _BORLANDC_
  extern unsigned _stklen=10000U;
#endif


#ifdef __ZTC__
  extern unsigned int _stack=10000U;
#endif


#define __DF1B2PART__
void df1b2_parameters::user_function(void)
{
  tau=exp(log_tau);
  nu=exp(log_nu);
  sigma=exp(log_sigma);
   df1b2variable Integral;
   int i;
   for (i=1;i<=k+1;i++)
   {
     a_index=i;
     Integral=adromb(-3.0,3.0,nsteps);
     S(i)=Integral;
     //S(i)=h(0);
   }
   f=0.0;
   for (i=1;i<=k;i++)
   {
     df1b2variable ff=0.0;
     df1b2variable diff=posfun((S(i)-S(i+1))/S(i),.000001,ff);
     f-=freq(i)*log(1.e-50+S(i)*diff);
     f+=ff;
   }
   f+=sum_freq*log(1.e-50+S(1));
   f+=0.5*norm2(u);
}

df1b2variable df1b2_parameters::adromb(double a,double b,int ns)
{
  const double base = 4;
  const int JMAX=50;
  df1b2variable ss=0.0;
  int MAXN = min(JMAX, ns);
  df1b2vector s(1,MAXN+1);
  for(int j=1; j<=MAXN+1; j++)
  {
    s[j] = trapzd(a,b,j,ss);
  }
  for(int iter=1; iter<=MAXN+1; iter++)
  {
    for(int j=1; j<=MAXN+1-iter; j++)
    {
      s[j] = (pow(base,iter)*s[j+1]-s[j])/(pow(base,iter)-1);
    }
  }
  return s[1];
}

df1b2variable df1b2_parameters::trapzd(double a,double b,int n,const df1b2variable& s)
{
  double x,num_interval,hn;
  df1b2variable sum;
 // static df1b2variable s;
  static int interval;
  int j;
  model_parameters * ptr= (model_parameters *) mycast();
  if (n == 1) {
    interval=1;
    return (s=0.5*(b-a)*(h(a)+h(b)));
  } else {
    num_interval=interval;
    hn=(b-a)/num_interval;
    x=a+0.5*hn;
    for (sum=0.0,j=1;j<=interval;j++,x+=hn) sum += h(x);
    interval *= 2;
    s=0.5*(s+(b-a)*sum/num_interval);
    return s;
  }
}

df1b2variable df1b2_parameters::h(const double z)
{
  df1b2variable tmp;
  tmp=exp(-.5*z*z + u(a_index)+ tau*(-1.+exp(-nu*pow(a(a_index),beta)*exp(sigma*z))) );  
  return tmp;
}
   
void df1b2_pre_parameters::setup_quadprior_calcs(void) 
{ 
  df1b2_gradlist::set_no_derivatives(); 
  quadratic_prior::in_qp_calculations=1; 
}  
  
void df1b2_pre_parameters::begin_df1b2_funnel(void) 
{ 
  (*re_objective_function_value::pobjfun)=0; 
  other_separable_stuff_begin(); 
  f1b2gradlist->reset();  
  if (!quadratic_prior::in_qp_calculations) 
  { 
    df1b2_gradlist::set_yes_derivatives();  
  } 
  funnel_init_var::allocate_all();  
}  
 
void df1b2_pre_parameters::end_df1b2_funnel(void) 
{  
  lapprox->do_separable_stuff(); 
  other_separable_stuff_end(); 
} 
  
void model_parameters::begin_df1b2_funnel(void) 
{ 
  if (lapprox)  
  {  
    {  
      begin_funnel_stuff();  
    }  
  }  
}  
 
void model_parameters::end_df1b2_funnel(void) 
{  
  if (lapprox)  
  {  
    end_df1b2_funnel_stuff();  
  }  
} 

void df1b2_parameters::allocate(void) 
{
  log_tau.allocate(-14,15,2,"log_tau");
  log_nu.allocate(-15,4,"log_nu");
  beta.allocate(.1,1.0,-1,"beta");
  log_sigma.allocate(-5,3,"log_sigma");
  tau.allocate("tau");
  nu.allocate("nu");
  sigma.allocate("sigma");
  S.allocate(1,k+1,"S");
  #ifndef NO_AD_INITIALIZE
    S.initialize();
  #endif
  f.allocate("f");  /* ADOBJECTIVEFUNCTION */
  u.allocate(1,k+1,"u");
}

/*
void df1b2_parameters::initializationfunction(void)
{
  log_tau.set_initial_value(0);
  beta.set_initial_value(0.6666667);
  log_nu.set_initial_value(0);
  log_sigma.set_initial_value(-2);
}
*/
